use std::ffi::{CStr, CString};
use std::os::raw::c_char;



/// Defines an empty string.
pub const EMPTY_STRING: &'static str = "";



/// Creates a new String from the data in the pointer.
///
/// Any conversion errors will result in an Empty
/// String being returned. Option is not used here for
/// simplicities sake.
pub fn from_c_string(c_string: *const c_char) -> String
{
   unsafe
   {
      CStr::from_ptr(c_string).to_string_lossy().into_owned()
   }
}

/// Creates a CString from a given str.
///
/// This may be "" if there is an UTF8 conversion error.
/// If it can not create an empty CString, then it will panic.
pub fn to_c_string<P>(string: P) -> CString
   where P: AsRef<str>
{
   // Try to create a CString from the given str.
   match CString::new(string.as_ref())
   {
      Ok(c_string) =>
      {
         // Return the new CString.
         c_string
      }

      Err(error) =>
      {
         warn!("Unable to create a CString from the given str: {}", error);

         // Try to atleast make an empty CString.
         match CString::new(EMPTY_STRING)
         {
            Ok(c_string) =>
            {
               // Return the new CString.
               c_string
            }

            Err(final_error) =>
            {
               // This is a show stopping error.
               error!("Unable to create an empty CString:  {}", final_error);
            }
         }
      }
   }
}
