/// Some times there are types that need to return a platform specific
/// data type that can be used by the platform specific functions. This
/// type creates a path to easily get that data.
pub trait AsRaw<T> where T: Sized
{
   fn as_raw(&self) -> &T;
}

/// Some times there are types that need to return a platform specific
/// data type that can be used by the platform specific functions. This
/// type creates a path to easily get that data in a mutable manner.
pub trait AsRawMut<T> where T: Sized
{
   fn as_raw_mut(&mut self) -> &mut T;
}

/// Some times there are types that need to return a platform specific
/// data type that can be used by the platform specific functions. This
/// type creates a path to easily get that data as a raw const pointer.
pub trait AsRawPtr<T> where T: Sized
{
   fn as_raw_ptr(&self) -> *const T;
}

/// Some times there are types that need to return a platform specific
/// data type that can be used by the platform specific functions. This
/// type creates a path to easily get that data as a raw mutable pointer.
pub trait AsRawMutPtr<T> where T: Sized
{
   fn as_raw_mut_ptr(&mut self) -> *mut T;
}

/// This makes it possible to create an object from a given piece of platform
/// specific data.
pub trait FromRaw<T>
{
   fn from_raw(raw: T) -> Self;
}

/// This makes it possible to convert an object into platform
/// specific data.
pub trait IntoRaw<T>
{
   fn into_raw(self) -> T;
}
